/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "frames.h"

#include <iostream>
#include <cmath>

#include <X11/Xlib.h>

#include <qpainter.h>
#include <qdatetime.h>
#include <qbitmap.h>
#include <qlayout.h>

#include <kwin.h>
#include <kdebug.h>
#include <klocale.h>
#include <krootpixmap.h>

MainFrame::MainFrame( QWidget *parent, const char *name )
	: QFrame(parent, name)
{
	tooltip = 0L;
	m_rootPixmap = 0L;
	
	m_currentIndex = 0;
	m_rateInLifo.resize(2000);
	m_rateOutLifo.resize(2000);
		
	config = KdevmonConfig::getConfig();
	
	m_graphPixmap = new GraphPixmap();
	
	m_rootPixmap = new KRootPixmap( this );
	
	initFromConfig();	
	clear();
}

MainFrame::~MainFrame()
{
}

void MainFrame::initFromConfig()
{
	if( config->getMainFrameConfig()->transparent )
	{
		m_rootPixmap->start();
		m_rootPixmap->repaint(true);
	}
	else
	{
		m_rootPixmap->stop();
		QPixmap p(size());
		p.fill( config->getMainFrameConfig()->bgColor );
		setBackgroundPixmap( p );
	}

	QColor bgColor = config->getMainFrameConfig()->bgColor;
	QPalette p(bgColor,bgColor);
	setPalette( p );
	
	if( config->getMainFrameConfig()->showTooltips )
	{
	    if( tooltip != 0 ) delete tooltip;
		tooltip = new RateTooltip(this, 0, true);
	}
	else
	{
		if( tooltip != 0 ) delete tooltip;
		tooltip = new RateTooltip(this, 0, false);
	}
	
	m_graphPixmap->initFromConfig();
	repaint();
}

void MainFrame::updateDisplay( double rateIn, double rateOut )
{
	// store rateIn/rateOut in lifo
	m_currentIndex++;
	m_currentIndex%=2000;
	m_rateInLifo[m_currentIndex] = rateIn;
	m_rateOutLifo[m_currentIndex] = rateOut;
	
	m_graphPixmap->drawGraph( m_rateInLifo, m_rateOutLifo, m_currentIndex );
	
	if( config->getMainFrameConfig()->transparent )
		repaint(true);
	else
		repaint(false);
}

void MainFrame::clear() {
	m_rateInLifo.fill(0);
	m_rateOutLifo.fill(0);
	m_currentIndex = 0;

	m_graphPixmap->drawGraph( m_rateInLifo, m_rateOutLifo, m_currentIndex );
	repaint();
}

QString MainFrame::getTooltipText( const QPoint& point ) const
{
	int w = width()-2*BORDERWIDTH;
	int diff = (int) (w - (double)point.x());
	double rIn =
		m_currentIndex-diff>=0 ?
		m_rateInLifo[m_currentIndex-diff]:
		m_rateInLifo[2000+m_currentIndex-diff];
	double rOut =
		m_currentIndex-diff>=0 ?
		m_rateOutLifo[m_currentIndex-diff] :
		m_rateOutLifo[2000+m_currentIndex-diff];
	
	int unit = config->getCommonConfig()->unit;
	int refresh = config->getCommonConfig()->refreshTime;
	
	QString s;
	int diffseconds = (int) ((double)diff * refresh / 1000);
	if( unit == Common::UNIT_BIT )
		s.sprintf( "%s \n In: %sbps | Out: %sbps",
			QTime::currentTime().addSecs(-diffseconds).toString().data(),
			Tools::double2bit(rIn).data(), Tools::double2bit(rOut).data() );
	else if( unit == Common::UNIT_BYTE )
		s.sprintf( "%s \n In: %sBps | Out: %sBps",
			QTime::currentTime().addSecs(-diffseconds).toString().data(),
			Tools::double2byte(rIn).data(), Tools::double2byte(rOut).data() );
	return s;
}

void MainFrame::resizeEvent( QResizeEvent *e )
{
	QFrame::resizeEvent( e );
	
	m_graphPixmap->setSize( width()-2*BORDERWIDTH, height()-2*BORDERWIDTH );
	repaint();
}

void MainFrame::paintEvent( QPaintEvent *e )
{
	QFrame::paintEvent( e );
	
	QPainter p;
	p.begin( this );
	p.drawPixmap( BORDERWIDTH, BORDERWIDTH, m_graphPixmap->getPixmap() );
	p.end();
}


DockFrame::DockFrame( QWidget *parent, const char *name )
	: QWidget(parent, name)
{
	m_currentIndex = 0;
	m_rateInLifo.resize(DOCK_WIDTH);
	m_rateOutLifo.resize(DOCK_WIDTH);
		
	config = KdevmonConfig::getConfig();
	
	m_graphPixmap = new GraphPixmap();
	
	initFromConfig();	
	clear();

	resize( DOCK_WIDTH, DOCK_WIDTH );
	KWin::setSystemTrayWindowFor( winId(), 0 );
}

DockFrame::~DockFrame() {
}

void DockFrame::initFromConfig()
{
	m_graphPixmap->initFromConfig();
	repaint(false);
}

void DockFrame::clear()
{
	m_rateInLifo.fill(0);
	m_rateOutLifo.fill(0);
	m_currentIndex = 0;

	m_graphPixmap->drawGraph( m_rateInLifo, m_rateOutLifo, m_currentIndex );
	repaint(false);
}

void DockFrame::updateDisplay( double rateIn, double rateOut )
{
	m_currentIndex++;
	m_currentIndex%=DOCK_WIDTH;
	m_rateInLifo[m_currentIndex] = rateIn;
	m_rateOutLifo[m_currentIndex] = rateOut;
	
	if( config->getDockFrameConfig()->showText )
	{
		QString textIn, textOut;
		if( config->getCommonConfig()->unit == Common::UNIT_BIT ) {
			textIn = Tools::double2bit( rateIn );
			textOut = Tools::double2bit( rateOut );
		}
		else if( config->getCommonConfig()->unit == Common::UNIT_BYTE ) {
			textIn = Tools::double2byte( rateIn );
			textOut = Tools::double2byte( rateOut );
		}

		QPixmap pm( DOCK_WIDTH, DOCK_WIDTH );
		pm.fill( config->getDockFrameConfig()->bgColor );
		QPainter painter;
		painter.begin( &pm );
		painter.setFont( QFont("times", 10) );
		painter.setPen( config->getCommonConfig()->colorIn );
		painter.drawText( 0, 22, textIn );
		painter.setPen( config->getCommonConfig()->colorOut );
		painter.drawText( 0, 11, textOut );
		painter.end();
		setBackgroundPixmap( pm );
	}
	else
	{
		QPixmap pm( DOCK_WIDTH, DOCK_WIDTH );
		pm.fill( config->getDockFrameConfig()->bgColor );
		setBackgroundPixmap( pm );
		
		m_graphPixmap->drawGraph(
			m_rateInLifo, m_rateOutLifo, m_currentIndex );
		repaint(false);
	}
}

void DockFrame::resizeEvent( QResizeEvent *e )
{
	QWidget::resizeEvent( e );
	
	m_graphPixmap->setSize( width(), height() );
	repaint(false);
}

void DockFrame::paintEvent( QPaintEvent *e )
{
	QWidget::paintEvent( e );
	
	if( ! config->getDockFrameConfig()->showText )
	{
		QPainter p;
		p.begin( this );
		p.drawPixmap( 0, 0, m_graphPixmap->getPixmap() );
		p.end();
	}
}


GraphPixmap::GraphPixmap()
	: QObject()
{
	m_graphPixmap = new QPixmap();
	m_mask = new QBitmap();
	
	m_lastIndex = 0;
	m_lastRateInArray = QArray<double>(0);
	m_lastRateOutArray = QArray<double>(0);
	
	initFromConfig();
}

void GraphPixmap::initFromConfig()
{
	KdevmonConfig *config = KdevmonConfig::getConfig();
	
	m_colorInArray[0]=config->getCommonConfig()->colorIn;
    for( int i=1; i<COLORS; i++ )
	    m_colorInArray[i]=m_colorInArray[i-1].dark(100+75/COLORS);
	m_colorOutArray[0]=config->getCommonConfig()->colorOut;
    for( int i=1; i<COLORS; i++ )
	  m_colorOutArray[i]=m_colorOutArray[i-1].dark(100+75/COLORS);
	
	drawGraph( m_lastRateInArray, m_lastRateOutArray, m_lastIndex );
}

void GraphPixmap::setSize( int w, int h )
{
	m_graphPixmap->resize( w, h );
	m_mask->resize( w, h );
	
	drawGraph( m_lastRateInArray, m_lastRateOutArray, m_lastIndex );
}

void GraphPixmap::drawGraph(
	const QArray<double>& rateInArray,
	const QArray<double>& rateOutArray,
	int currentIndex )
{
	if( (m_graphPixmap->isNull()) ||
		((int) rateInArray.size() < m_graphPixmap->width()) )
		return;
	
	KdevmonConfig *conf = KdevmonConfig::getConfig();
	QPainter painter;
	
	static QArray<int> lengthInArray(0);
	static QArray<int> lengthOutArray(0);
	int
		w = m_graphPixmap->width(),
		h = m_graphPixmap->height();
	int i;
	if( (int) lengthInArray.size() != w )
	{
		lengthInArray.resize(w);
		lengthOutArray.resize(w);
	}
	lengthInArray.fill(0);
	lengthOutArray.fill(0);
	
	// calculate length of every bar of the graph
	double scaling;
	if( conf->getGraphConfig()->scalingMode == Common::MODE_CBRT )
		scaling = cbrt(conf->getGraphConfig()->scaling / 8.0);
	else
		scaling = conf->getGraphConfig()->scaling / 8.0;

	double rateIn, rateOut;
	for( int i=0; i<w; i++ )
	{
		int index = currentIndex-i<0 ? w+currentIndex-i : currentIndex-i;
		if( conf->getGraphConfig()->scalingMode == Common::MODE_CBRT )
		{
			rateIn = cbrt( rateInArray[index] );
			rateOut = cbrt( rateOutArray[index] );
		}
		else
		{
			rateIn = rateInArray[index];
			rateOut = rateOutArray[index];
		}
		double maxScale = scaling;
		if ( conf->getGraphConfig()->graphPosition == Common::POS_BOTTOM )
		{
			if (rateIn > maxScale) maxScale = rateIn;
			if (rateOut > maxScale) maxScale = rateOut;
		}
		else if ((rateIn + rateOut) > maxScale) maxScale = rateIn + rateOut;

		lengthInArray[w-1-i] = (int) (rateIn / maxScale * h);
		lengthOutArray[w-1-i] = (int) (rateOut / maxScale * h);
	}

	// reset pixmap
	m_graphPixmap->fill( conf->getMainFrameConfig()->bgColor );
	
	// reset mask
	m_mask->fill( Qt::color0 );
	
	// draw grid	
	int space = getGridRowHeight();
	
	painter.begin( m_graphPixmap );
	painter.setPen( conf->getMainFrameConfig()->gridColor );
	for( i = 0; i < h; i+=space )
		painter.drawLine(0, h-i, w, h-i);
	painter.end();
	
	painter.begin( m_mask );
	painter.setPen( Qt::color1 );
	for( i = 0; i < h; i+=space )
		painter.drawLine(0, h-i, w, h-i);
	painter.end();
	
	// draw graph
	painter.begin( m_graphPixmap );
	if( conf->getGraphConfig()->graphStyle == Common::STYLE_LINES )
	{
		painter.setPen( QPen(m_colorInArray[0], 1, SolidLine) );
		for( i=1; i<w; i++ )
			if( (lengthInArray[i]!=0) || (lengthInArray[i-1]!=0) )
				painter.drawLine(
					i-1,
					h-lengthInArray[i-1],
					i,
					h-lengthInArray[i] );

		painter.setPen( QPen(m_colorOutArray[0], 1, SolidLine) );
		for( i=1; i<w; i++ )
			if( (lengthOutArray[i]!=0) || (lengthOutArray[i-1]!=0) )
			{
				if( conf->getGraphConfig()->graphPosition ==
					Common::POS_BOTTOM )
					painter.drawLine(
						i-1,
						h-lengthOutArray[i-1],
						i,
						h-lengthOutArray[i] );
				else if ( conf->getGraphConfig()->graphPosition ==
					Common::POS_TOP_BOTTOM )
					painter.drawLine(
						i-1,
						lengthOutArray[i-1],
						i,
						lengthOutArray[i] );
			}
	}
	else if (conf->getGraphConfig()->graphStyle == Common::STYLE_SHADED )
	{
		for( int color = 0; color < COLORS; color++ )
		{
			painter.setPen( m_colorInArray[color] );
			for( i=0; i<w; i++ )
				if( lengthInArray[i] != 0 )
					painter.drawLine(
						i,
						h - lengthInArray[i]*(COLORS-color)/COLORS,
						i,
						h );
		}

		for( int color = 0; color < COLORS; color++ )
		{
			painter.setPen( m_colorOutArray[color] );
			for( i=0; i<w; i++ )
				if( lengthOutArray[i] != 0 )
				{
					if( conf->getGraphConfig()->graphPosition ==
						Common::POS_TOP_BOTTOM )
						painter.drawLine(
							i,
							0,
							i,
							lengthOutArray[i]*(COLORS-color)/COLORS );
					else if ( conf->getGraphConfig()->graphPosition ==
						Common::POS_BOTTOM )
						painter.drawLine(
							i,
							h - lengthOutArray[i]*(COLORS-color)/COLORS,
							i,
							h);
				}
		}

		if ( conf->getGraphConfig()->graphPosition ==
			Common::POS_BOTTOM )
		{
			painter.setPen( QPen(m_colorInArray[0], 1, SolidLine) );
			for( i=1; i<w; i++ )
				if(((lengthOutArray[i] > lengthInArray[i]) ||
					(lengthOutArray[i - 1] > lengthInArray[i - 1])) &&
					(lengthInArray[i] != 0) )
					painter.drawLine(
						i-1,
						h-lengthInArray[i-1],
						i,
						h-lengthInArray[i] );
		}

	}
	else if ( conf->getGraphConfig()->graphStyle == Common::STYLE_SOLID )
	{
		painter.setPen( m_colorInArray[0] );
		for( i=0; i<w; i++ )
			if( lengthInArray[i] != 0 )
				painter.drawLine(
					i,
					h - lengthInArray[i],
					i,
					h );

		painter.setPen( m_colorOutArray[0] );
		for( i=0; i<w; i++ )
			if( lengthOutArray[i] != 0 )
			{
				if( conf->getGraphConfig()->graphPosition ==
					Common::POS_TOP_BOTTOM )
					painter.drawLine(
						i,
						0,
						i,
						lengthOutArray[i] );
				else if ( conf->getGraphConfig()->graphPosition ==
					Common::POS_BOTTOM )
					painter.drawLine(
						i,
						h - lengthOutArray[i],
						i,
						h);
			}
		if ( conf->getGraphConfig()->graphPosition ==
			Common::POS_BOTTOM )
		{
			painter.setPen( QPen(m_colorInArray[0], 1, SolidLine) );
			for( i=1; i<w; i++ )
				if(((lengthOutArray[i] > lengthInArray[i]) ||
					(lengthOutArray[i - 1] > lengthInArray[i - 1])) &&
					(lengthInArray[i] != 0) )
					painter.drawLine(
						i-1,
						h-lengthInArray[i-1],
						i,
						h-lengthInArray[i] );
		}
	}
	painter.end();
	
	painter.begin( m_mask );
	painter.setPen( Qt::color1 );
	
	if( conf->getGraphConfig()->graphStyle == Common::STYLE_LINES )
	{
		for( i=1; i<w; i++ )
			if( (lengthInArray[i]!=0) || (lengthInArray[i-1]!=0) )
				painter.drawLine(
					i-1,
					h - lengthInArray[i-1],
					i,
					h - lengthInArray[i] );

		for( i=1; i<w; i++ )
			if( (lengthOutArray[i]!=0) || (lengthOutArray[i-1]!=0) )
			{
				if( conf->getGraphConfig()->graphPosition ==
					Common::POS_BOTTOM )
					painter.drawLine(
						i-1,
						h - lengthOutArray[i-1],
						i,
						h - lengthOutArray[i] );
				else if ( conf->getGraphConfig()->graphPosition ==
					Common::POS_TOP_BOTTOM )
					painter.drawLine(
						i-1,
						lengthOutArray[i-1],
						i,
						lengthOutArray[i] );
			}
	}
	else
	{
		for( i=0; i<w; i++ )
			if( lengthInArray[i]!=0 )
				painter.drawLine(
					i,
					h - lengthInArray[i],
					i,
					h );

		for( i=0; i<w; i++ )
			if( lengthOutArray[i] != 0 )
			{
				if( conf->getGraphConfig()->graphPosition ==
					Common::POS_TOP_BOTTOM )
					painter.drawLine(
						i,
						0,
						i,
						lengthOutArray[i] );
				else if ( conf->getGraphConfig()->graphPosition ==
					Common::POS_BOTTOM )
					painter.drawLine(
						i,
						h - (lengthInArray[i]+lengthOutArray[i]),
						i,
						h - lengthInArray[i] );
			}
	}
	painter.end();
	
	// if frame is not transparent, avoid flickering by not setting a mask.
	// transparent frames still flicker (dont know how I can fix it)
	if( conf->getMainFrameConfig()->transparent )
		m_graphPixmap->setMask( *m_mask );
	else
		m_graphPixmap->setMask( QBitmap() );
}

int GraphPixmap::getGridRowHeight()
{
	KdevmonConfig *conf = KdevmonConfig::getConfig();	

	int h = m_graphPixmap->height() - 1;
	int preferredHeight = h < 100 ? 10 : 20;
	double scaling = conf->getGraphConfig()->scaling / 8.0;

	double tries[6];
	tries[0] = pow(10, (int)log10(scaling)-1);
	tries[1] = 2*tries[0];
	tries[2] = 5*tries[0];
	tries[3] = 10*tries[0];
	tries[4] = 20*tries[0];
	tries[5] = 50*tries[0];
	
	double error = 1000;
	double tmp;
	int found = 0;
	int i;
	for( i = 0; i < 6; i++ )
		if( (tmp=fabs((tries[i]/scaling*h)-preferredHeight)) < error )
		{
			found = i;
			error = tmp;
		}
		
	return (int) fabs(tries[found]/scaling*h);
}


BpsFrame::BpsFrame ( QWidget *parent, const char *name )
	: QFrame ( parent, name )
{
	font = QFont( "times" );
	pixmap = new QPixmap( width(), height() );
	pixmapColor = Qt::black;
	pixmap->fill( pixmapColor );
}

BpsFrame::~BpsFrame()
{}

void BpsFrame::resizeEvent( QResizeEvent * )
{
	this->font = QFont( "times", width()/4<height() ? width()/4 : height() );
	
	pixmap->resize( size() );
	pixmap->fill( pixmapColor );
}

void BpsFrame::setPixmapColor( const QColor& color )
{
	this->pixmapColor = color;
	this->pixmap->fill( color );
}

void BpsFrame::drawText( const QString& text, const QColor& color )
{
	QPainter painter;
	painter.begin( this->pixmap );
	painter.fillRect( rect(), this->pixmapColor );
	painter.setPen( color );
	painter.setFont( this->font );
	painter.drawText( rect(), AlignCenter, text );
	painter.end();
	
	setBackgroundPixmap( *pixmap );
}


BarFrame::BarFrame( QWidget *parent, const char* name )
	: QFrame ( parent, name )
{
	pixmapColor = Qt::black;
	colorIn = Qt::black;
	colorOut = Qt::black;
	this->pixmap = new QPixmap( width(), height() );	
	this->pixmap->fill();
	this->lastPercentIn = 0; this->lastPercentOut = 0;
}

BarFrame::~BarFrame()
{}

void BarFrame::resizeEvent( QResizeEvent * )
{
	pixmap->resize( size() );	
	updatePixmap( this->lastPercentIn, this->lastPercentOut );
}

void BarFrame::updatePixmap( double percentIn, double percentOut )
{
	QPainter painter;
	painter.begin( this->pixmap );
	painter.fillRect( 0, 0, this->pixmap->width(), this->pixmap->height(), this->pixmapColor );
	painter.setPen( this->pixmapColor );
	painter.setBrush( this->colorIn );
	for( int i = 0; i < this->pixmap->width()*percentIn; i+=this->pixmap->width()/10 )
		painter.drawRect( this->pixmap->width()-i, 0,
			this->pixmap->width()/10, this->pixmap->height()/2 );
	painter.setBrush( this->colorOut );
	for( int i = 0; i < this->pixmap->width()*percentOut; i+=this->pixmap->width()/10 )
		painter.drawRect(this->pixmap->width()-i, this->pixmap->height()/2,
			this->pixmap->width()/10, this->pixmap->height()/2);
	painter.end();
	
	setBackgroundPixmap( *pixmap );
}

const QColor& BarFrame::getPixmapColor() { return this->pixmapColor; }

void BarFrame::setPixmapColor( const QColor& color )
{
	this->pixmapColor = color;
	this->pixmap->fill( color );
	this->updatePixmap( this->lastPercentIn, this->lastPercentOut );
}

void BarFrame::setColorIn( const QColor& color )
{
	this->colorIn = color;
	this->updatePixmap( this->lastPercentIn, this->lastPercentOut );
}

void BarFrame::setColorOut( const QColor& color )
{
	this->colorOut = color;
	this->updatePixmap( this->lastPercentIn, this->lastPercentOut );
}

void BarFrame::clear()
{
	this->lastPercentIn = 0; this->lastPercentOut = 0;
	this->updatePixmap( 0, 0 );
}


RateTooltip::RateTooltip( QWidget *parent, QToolTipGroup *group, bool show )
	: QToolTip( parent, group ), showTooltips( show )
{}

void RateTooltip::maybeTip( const QPoint& point )
{
	QRect r( point.x(), point.y(), 1, 1 );
	
	QString s = ((MainFrame*)parentWidget())->getTooltipText( point );
	
	if( showTooltips )
		tip( r, s );
}

void RateTooltip::setShowTooltips( bool show ) { showTooltips = show; }


QString Tools::double2bit( double value )
{
	QString s;
	
	value *= 8;

	if( value > 1000000000 ) s.sprintf("%1.3fg", value/1000000000);
	else if( value > 1000000 ) s.sprintf("%2.2fm", value/1000000);
	else if ( value > 1000 ) s.sprintf("%3.1fk", value/1000);
	else s.sprintf("%4.0f", value);

	return s;
}

QString Tools::double2byte( double value )
{
	QString s;
	
	if( value > 1073741824 ) s.sprintf("%1.3fG", value/1073741824);
	else if( value > 1048576 ) s.sprintf("%2.2fM", value/1048576);
	else if ( value > 1024 ) s.sprintf("%3.1fK", value/1024);
	else s.sprintf("%4.0f", value);

	return s;
}
