/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FRAMES_H
#define FRAMES_H

#define BORDERWIDTH 2
#define COLORS 10
#define DOCK_WIDTH 24

#include "resources.h"
#include "kdevmonconfig.h"

#include <qframe.h>
#include <qcolor.h>
#include <qtooltip.h>
#include <qarray.h>

class BpsFrame;
class RateTooltip;
class GraphPixmap;
class QPixmap;
class QBitmap;
class KRootPixmap;

/** class for a frame that shows a graph of two specified values.
 * (for example the values can be the in/out rate from a network device).
 */
class MainFrame : public QFrame
{
	Q_OBJECT
public:
	MainFrame( QWidget *parent=0, const char *name=0 );
	~MainFrame();
	
	void initFromConfig();
	
	void updateDisplay( double rateIn, double RateOut );
	void clear();
	QString getTooltipText( const QPoint& point ) const;

protected:
	void paintEvent( QPaintEvent * );
	void resizeEvent( QResizeEvent * );
	
private:
	QArray<double> m_rateInLifo;
	QArray<double> m_rateOutLifo;
	int m_currentIndex;
	
	GraphPixmap *m_graphPixmap;
	KRootPixmap *m_rootPixmap;
	RateTooltip *tooltip;
	KdevmonConfig *config;
};


class DockFrame : public QWidget
{
	Q_OBJECT
public:
	DockFrame( QWidget *parent=0, const char *name=0 );
	~DockFrame();
	
	void initFromConfig();
	
	void updateDisplay( double rateIn, double RateOut );
	void clear();
	void setShowText( bool show );

protected:
	void paintEvent( QPaintEvent * );
	void resizeEvent( QResizeEvent * );
	
private:
	QArray<double> m_rateInLifo;
	QArray<double> m_rateOutLifo;
	int m_currentIndex;
	GraphPixmap *m_graphPixmap;
	KdevmonConfig *config;

};


class GraphPixmap : public QObject
{
public:	
	GraphPixmap();
	
	void initFromConfig();
	void drawGraph( const QArray<double>&, const QArray<double>&, int );
	void setSize( int w, int h );
	const QPixmap &getPixmap() const { return *m_graphPixmap; }
	
private:
	QPixmap *m_graphPixmap;
	QBitmap *m_mask;
	QColor
		m_colorInArray[COLORS],
		m_colorOutArray[COLORS];
	QArray<double>
		m_lastRateInArray,
		m_lastRateOutArray;
	int m_lastIndex;
	
	int getGridRowHeight();
};


/** class for a frame that simply draws a text in a specified color
 */
class BpsFrame : public QFrame {
public:	
	/** constructor, only sets font */
	BpsFrame( QWidget *parent, const char *name );
	~BpsFrame();
	
	/** draws text */
	void drawText( const QString& text, const QColor& color);
	/** change background color */
	void setPixmapColor( const QColor& color );
protected:
	/** resizes font */
	void resizeEvent( QResizeEvent * );	
private:
	QFont font;
	QColor pixmapColor;
	QPixmap *pixmap;
};


/** very simple class for a frame that only draws a bar.
 * Placeholder for a more complex class
 */
class BarFrame : public QFrame {
public:
	/** constructor, sets up pixmap on which the circle is drawn */
	BarFrame( QWidget *parent, const char* name );
	~BarFrame();
	
	/** draws circle on pixmap */
	void updatePixmap( double rateIn, double rateOut );
	/** get color of pixmap on which the circle is drawn */
	const QColor& getPixmapColor();
	/** change background color */
	void setPixmapColor( const QColor& color );
	/** sets color for graph of incoming bytes */
	void setColorIn( const QColor& color );
	/** sets color for graph of outgoing bytes */
	void setColorOut( const QColor& color );
	/** clears pixmap of barframe */
	void clear();
protected:
	/** resizes pixmap */
	void resizeEvent( QResizeEvent * );
private:
	QPixmap *pixmap;
	QColor
	    pixmapColor,
	    colorIn,
	    colorOut;
	double lastPercentIn, lastPercentOut;
};


class RateTooltip : public QToolTip {
public:
	RateTooltip( QWidget *parent=0, QToolTipGroup *group=0, bool show=true );
	
	void setShowTooltips( bool show );
protected:
	void maybeTip( const QPoint& point );
private:
	bool showTooltips;
};

class Tools
{
public:
	static QString double2bit( double value );
	static QString double2byte( double value );
};

#endif
